#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import os, string, sys
import math
from array import array
#Author: Tin Tran
#bezier helper functions tested in codeskulptor.org
#http://www.codeskulptor.org/#user41_3ejFX0pINN_6.py

#c=(17.0, 91.0, 52.0, 65.0, 91.0, 35.0, 163.0, 166.0, 274.66666666666663, 99.66666666666666, 390.0, 39.0, 515.0, 87.0, 536.0, 204.0, 551.0, 270.0)
def control_points_to_beziers(controls):
    usepoints = controls[2:len(controls)-2]
    i = 0
    beziers = []
    while i+7 <= len(usepoints):
        bezier = [(usepoints[i],usepoints[i+1]),
                  (usepoints[i+2],usepoints[i+3]),
                  (usepoints[i+4],usepoints[i+5]),
                  (usepoints[i+6],usepoints[i+7])]
        beziers.append(bezier)
        i += 6
		
    return (beziers)
def control_points_to_straightline(controls):
    usepoints = controls[2:len(controls)-2]
    return (((usepoints[0],usepoints[1]),(usepoints[len(usepoints)-2],usepoints[len(usepoints)-1])))

#gets distance between 2 points
def distance(p0,p1):
    dist = ((p1[0]-p0[0])**2 + (p1[1]-p0[1])**2)**0.5
    return (dist)
#gets the point on line given t as time fraction of whole line
def get_point_on_line(p0,p1,t):
    x = p0[0] + (p1[0] - p0[0]) * t
    y = p0[1] + (p1[1] - p0[1]) * t
    return ((x,y))
	
def get_straightlight_points(line,sections):
    points = [line[0]]
    for i in range(1,sections+1):
        p_next = get_point_on_line(line[0],line[1],i*1.0/sections)
        points.append(p_next)
    return points        

#gets point on cubic bezier curve given t as time 
#0<=t<=1 is fraction of whole curve
#based on this reference image http://flower2fruit.blogspot.ca/2016/05/cubic-bezier-curve-image-reference.html
def get_bezier_point(bezier,t):
    p0 = bezier[0]
    p1 = bezier[1]
    p2 = bezier[2]
    p3 = bezier[3]
    green_p0 = get_point_on_line(p0,p1,t)
    green_p1 = get_point_on_line(p1,p2,t)
    green_p2 = get_point_on_line(p2,p3,t)
    blue_p0 = get_point_on_line(green_p0,green_p1,t)
    blue_p1 = get_point_on_line(green_p1,green_p2,t)
    point = get_point_on_line(blue_p0,blue_p1,t)
    return (point)

#get approximate bezier length when divided into sections of straight lines
def get_bezier_length(bezier,sections):
    dist1 = 0.0
    for i in range(0,sections):
        p0 = get_bezier_point(bezier,i*1.0/sections)
        p1 = get_bezier_point(bezier,(i+1)*1.0/sections)
        dist1 += distance(p0,p1)
    return (dist1)
def get_beziers_length(beziers,sections):
    dist0 = 0.0
    for bezier in beziers:
        dist0 += get_bezier_length(bezier,sections)
    return (dist0)
#get point on group of beziers give t, t is a fraction from 0 to 1 over the whole group of beziers
def get_beziers_point(beziers,t):
    
    total_dist = get_beziers_length(beziers,255)
    
    target_dist = t * total_dist  #the target distance we're looking for
    dist = 0.0
    
    for bezier in beziers:
        bezier_dist = get_bezier_length(bezier,255)
        if (dist <= target_dist) and (dist + bezier_dist >= target_dist): #we found it it's within this range
            difference_dist = target_dist - dist
            if bezier_dist == 0.0: #division by zero check
                bezier_t = 0.0
            else: 
                bezier_t = difference_dist*1.0/bezier_dist
            return (get_bezier_point(bezier,bezier_t))
        dist+= bezier_dist #keep adding distance to our dist to search for target_dist

	#we should never hit this part of the code
    pdb.gimp_message("target_dist not found in get_beziers_point()")
    return (None)
def get_beziers_points(beziers,sections):
    
    points = [beziers[0][0]]
    
    for i in range(1,sections+1):
        p_next = get_beziers_point(beziers,i*1.0/sections)
        points.append(p_next)
    return (points)

#get pixel helper function
def get_pixel(point,image_width,p_size,src_pixels) :
	x = int(point[0])
	y = int(point[1])
	src_pos = (x + image_width * y) * p_size
	pix = src_pixels[src_pos: src_pos + p_size]
	if p_size == 3:    #if it doesn't have alpha information add full opacity
		pix.append(255)
	return (pix)
#helper function to convert a list of bytes to color tuple
def color(pixeldata):
	return ((pixeldata[0],pixeldata[1],pixeldata[2]))

def my_gradient(image,layer,gradient_name,segments,straight):
	segments = int(segments)
	pdb.gimp_layer_resize_to_image_size(layer) # make layer same as image size to not deal with possible layer offsets
	
	vector = pdb.gimp_image_get_active_vectors(image)
	
	ret_value = pdb.gimp_vectors_get_strokes(vector)
	strokes = ret_value[0]
	strokeids = ret_value[1]
	
	for strokeid in strokeids:
		ret_value = pdb.gimp_vectors_stroke_get_points(vector,strokeid)
		stroke_type = ret_value[0]
		num_points = ret_value[1]
		controlpoints = ret_value[2]
		closed = ret_value[3]
		
		#debug for outputing to codeskulptor file testing
		#http://www.codeskulptor.org/#user41_BXdoMHid3zwluza_1.py
		# paste the output of the below into codeskulptor program in the above link 
		pdb.gimp_message("c=" + str(controlpoints))
		
		#grab bezier points information by doing math with helper functions
		if not straight:
			beziers = control_points_to_beziers(controlpoints) #get list of beziers
			beziers_points = get_beziers_points(beziers,segments)	#divide it into number of segments with (segments+1) number of points
		else:
			beziers = control_points_to_straightline(controlpoints)
			beziers_points = get_straightlight_points(beziers,segments)
			pdb.gimp_message("Straight line calculation even distance")
		
		
		#grab layer's pixel information
		srcRgn = layer.get_pixel_rgn(0, 0, layer.width, layer.height,False, False)
		src_pixels = array("B", srcRgn[0:layer.width, 0:layer.height])
		p_size = len(srcRgn[0,0])
		pixels = []     #save pixels data at bezier points
		for point in beziers_points:
			pix = get_pixel(point,layer.width,p_size,src_pixels)
			pixels.append(list(pix))
		#creates new gradient
		gradient_name = pdb.gimp_gradient_new(gradient_name)
		#delete 1 default segment because we'll add number of segments back
		for i in range(2,segments+1):
			pdb.gimp_gradient_segment_range_split_midpoint(gradient_name,0,0) #splits segment to add new segments
		pdb.gimp_gradient_segment_range_redistribute_handles(gradient_name,0,-1) #redistribute gradient uniformly
		
		#pdb.gimp_gradient_segment_range_set_coloring_type(gradient_name,0,-1,0)
		
		#sets gradient colors based on sample pixels data
		for i in range(0,segments):
			pdb.gimp_gradient_segment_set_left_color(gradient_name,i,color(pixels[i][0:3]),pixels[i][3]*1.0/255.0*100)
			pdb.gimp_gradient_segment_set_right_color(gradient_name,i,color(pixels[i+1][0:3]),pixels[i+1][3]*1.0/255.0*100)
			
	
register(
    "python_fu_create_new_gradient_along_path",
    "Creates new gradient by sampling pixels on active layer along active path",
    "Creates new gradient by sampling pixels on active layer along active path",
    "Tin Tran",
    "GPL",
    "May 2016",
    "<Image>/Tools/Create New Gradient Along Path...",
    "*",
    [
    #(PF_IMAGE, "image", "Input image", None), #current image
    #(PF_DRAWABLE, "layer", "Input layer", None), #current layer
    #(PF_FILENAME, "curvefilename","Curfile", dircurve ),
	(PF_STRING,  "gradient_name", "New Gradient Name:", "New Gradient"),
	(PF_ADJUSTMENT, "segments", "Number Of Segments in Gradient:", 7, (2, 1024, 1)),
	(PF_TOGGLE, "straight",   "Straight Line Between 2 points:", 1)
    ],
    [],
    my_gradient)

main()